#!/config/sh                              # just and only for syntax highlighting
#********************************************************************************
# qware build environment - header file of the qube configure script            *
#                                                                               *
#  Author: Kai Peter, ©2020-present                                             *
# Version: 0.6                                                                  *
# License: ISC                                                                  *
#                                                                               *
# Description: Standard commands and functions of the qube configure process.   *
#              Only developers should adapt it to their package requirements.   *
#********************************************************************************
declare -r QUBEDIR="build.in"     # MUST match with the first line in 'configure'
cd `dirname $0`
#----- default functions (basic) ------------------------------------------------
# 
opts() { [ -f "configure.opts" ] && source ./configure.opts || return; }
# if the file 'configure.help' doesn't exists, exit silently (ignore it)
help() { [ -f "configure.help" ] && source ./configure.help || exit 1; }
# general configure error handling: print message given by "$1" and exit
error() { [ "$1" ] && [ -n "$1" ] && printf "\033[91m$1!\033[0m\n" && exit 1; }
# just print a message for disabled core functions
disabled() { echo " disabled (OK)"; }

getSystype() {
  echo -n "Checking system ... "
  local OS=`uname -s | tr '[A-Z]' '[a-z]'`
  echo "detected $OS!"
  echo -n "Checking architecture ... "
  local ARCH=`uname -m`
  echo "$OS:$ARCH" > systype
  echo "build on $ARCH!"
}

mkShebang() {
  QUBE_SHEBANG=`which env` || exit 1   # an error shouldn't happen, but who knows
  echo "#!$QUBE_SHEBANG bash" > ac-warn.sh   # output was formerly 'warn-auto.sh'
  echo "# !!! WARNING: This file was auto-generated. Do NOT edit!" >> ac-warn.sh
  # Hint: To use a different interpreter than 'sh' you can do something like this
  #       in a Makefile recipe (just an example to get the idea):
  #
  #       cat ac-warn.sh | sed 's/ sh$/ perl/g' | ...

  return
}

#**** begin processing **********************************************************
case $1 in --help|-h) help;; esac      # fallback: 'configure.opts' was not found
case $1 in *) opts;; esac              #

make clean

# get variables from 'conf-build' and corresponding 'conf-*' files **************
source "$QUBEDIR/conf-build"           # required and must be present in any case
[ -f conf-build  ] && . ./conf-build                    # maybe a packagers patch
[ -f conf-prefix ] && PREFIX=`head -1 conf-prefix`      # maybe a packagers patch
[ -f conf-suffix ] && SUFFIX=`head -1 conf-suffix`      # maybe a packagers patch

# include the files with qube standard functions ...
F="$QUBEDIR/get-layout"    ; [ -f "$F" ] && source "$F"
F="$QUBEDIR/get-usrcreds"  ; [ -f "$F" ] && source "$F"
F="$QUBEDIR/get-externals" ; [ -f "$F" ] && source "$F"
F="$QUBEDIR/mk-buildtools" ; [ -f "$F" ] && source "$F"
F="$QUBEDIR/set-sysdeps"   ; [ -f "$F" ] && source "$F"
unset F
# ... and execute the qube standard functions
[ ! "$SYSTYPE" ] && getSystype
[ `type -t getLayout` ]    && getLayout
[ `type -t getUsrCreds` ]  && getUsrCreds
[ `type -t chkExtRes` ]    && chkExtRes
[ `type -t mkBuildTools` ] && mkBuildTools
[ `type -t setSysDeps` ]   && setSysDeps
. "$QUBEDIR"/mk-autotpl

# The last step: create 'ac-warn.sh' always *************************************
# Usually 'ac-warn.sh is required in 99.9% of build cases anyway, otherwise could
# be used in the Makefile to check if the configure script was invoked already.
[ -f ac-warn.sh ] || mkShebang
